﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using Syncfusion.Windows.Controls.Gantt;
using Syncfusion.Windows.Controls.Gantt.Chart;
using System.Windows.Controls.Primitives;
using System.Diagnostics;
using System.Windows.Documents;
using System.Collections.Generic;
using Syncfusion.Windows.Controls.Gantt.Schedule;

namespace ResourceView
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            this.Gantt.NodeDragCompleted += Gantt_NodeDragCompleted;
            this.Gantt.NodeResizingCompleted += Gantt_NodeResizingCompleted;
        }

        void Gantt_NodeResizingCompleted(object sender, NodeResizingEventArgs args)
        {
            
            args.Handled = PreventOverlap(args);
        }

        void Gantt_NodeDragCompleted(object sender, NodeDragAndDropEventArgs args)
        {
            args.Handled = PreventOverlap(args);
        }

        private bool PreventOverlap(RoutedEventArgs args)
        {
            
            GanttNode node = null;
            DateTime startTime;
            DateTime endTime;
            GanttChartRowItemsPresenter itemsPresenter = null;

            if (args is NodeResizingEventArgs)
            {
                var arguments = args as NodeResizingEventArgs;
                node = arguments.Node;
                itemsPresenter = arguments.NodePresenter;
                startTime = arguments.StartTime;
                endTime = arguments.EndTime;
            }
            else
            {
                var arguments = args as NodeDragAndDropEventArgs;
                node = arguments.Node;
                itemsPresenter = arguments.NodePresenter;
                startTime = arguments.StartTime;
                endTime = arguments.EndTime;
            }
           
            List<GanttNode> nodeCollection = new List<GanttNode>();

            //Getting the collection of GanttNodes in same row.
            foreach (UIElement child in itemsPresenter.Children)
            {
                if (child is GanttNode)
                {
                    GanttNode ganttNode = child as GanttNode;
                    nodeCollection.Add(ganttNode);
                }
            }

            //removing the current node from the collection.
            nodeCollection.Remove(node);

            for (int i = 0; i < nodeCollection.Count; i++)
            {
                //Checking whether the node is overlapping the another node while Drag and drop or resizing the node.
                if (nodeCollection[i].StartTime.CompareTo(startTime) >= 0 && nodeCollection[i].EndTime.CompareTo(endTime) <= 0)
                {
                    return true;
                }
                else if (endTime.CompareTo(nodeCollection[i].StartTime) >= 0 && endTime.CompareTo(nodeCollection[i].EndTime) <= 0)
                {
                    return true;
                }
                else if (startTime.CompareTo(nodeCollection[i].StartTime) >= 0 && startTime.CompareTo(nodeCollection[i].EndTime) <= 0)
                {
                    return true;
                }
            }
            return false;
        }
    }
}
