﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Collections.Specialized;
using Syncfusion.Windows.Controls.Gantt;

namespace ResourceView
{
    class ViewModel : INotifyPropertyChanged
    {
        private ObservableCollection<Item> _ganttItemSource;

        /// <summary>
        /// Initializes a new instance of the <see cref="ViewModel"/> class.
        /// </summary>
        public ViewModel()
        {
            _ganttItemSource = ViewModel.GetTeamInfo();
        }

        /// <summary>
        /// Gets or sets the appointment item source.
        /// </summary>
        /// <value>The appointment item source.</value>
        public ObservableCollection<Item> GanttItemSource
        {
            get
            {
                return _ganttItemSource;
            }
            set
            {
                _ganttItemSource = value;
            }
        }

        #region INotifyPropertyChanged Members

        private void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                var e = new PropertyChangedEventArgs(propertyName);
                handler(this, e);
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        /// <summary>
        /// Gets the team info.
        /// </summary>
        /// <returns></returns>
        public static ObservableCollection<Item> GetTeamInfo()
        {
            DateTime dtS = DateTime.Today;

            ObservableCollection<Item> teams = new ObservableCollection<Item>();

            teams.Add(new Item() { Name = "RDU Team" });
            Item Person = new Item() { Name = "Robert" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 07), FinishDate = new DateTime(2012, 01, 11), Name = "Market Analysis", Progress = 50d });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 11), FinishDate = new DateTime(2012, 01, 15), Name = "Competitor Analysis", Progress = 20d });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 15), FinishDate = new DateTime(2012, 01, 19), Name = "Desing Spec" });
            teams[0].SubItems.Add(Person);

            Person = new Item() { Name = "Michael" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 18), FinishDate = new DateTime(2012, 01, 19), Name = "Basic Requirement Analysis", Progress = 40 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 19), FinishDate = new DateTime(2012, 01, 21), Name = "Requirement Spec" });
            teams[0].SubItems.Add(Person);

            Person = new Item() { Name = "Anne" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 21), FinishDate = new DateTime(2012, 01, 24), Name = "Estimation", Progress = 30 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 24), FinishDate = new DateTime(2012, 01, 26), Name = "Budget & Plan Spec" });
            teams[0].SubItems.Add(Person);


            teams.Add(new Item() { Name = "Graphics Team" });
            Person = new Item() { Name = "Madhan" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 18), FinishDate = new DateTime(2012, 01, 20), Name = "Identifying UI modules", Progress = 40 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 21), FinishDate = new DateTime(2012, 01, 25), Name = "Defining UI Design" });
            teams[1].SubItems.Add(Person);

            Person = new Item() { Name = "Peter" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 21), FinishDate = new DateTime(2012, 01, 24), Name = "Designing Animagions", Progress = 40 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 25), FinishDate = new DateTime(2012, 01, 27), Name = "Completing Overall Graphics design" });
            teams[1].SubItems.Add(Person);


            teams.Add(new Item() { Name = "Dev Team" });
            Person = new Item() { Name = "Ruban" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 21), FinishDate = new DateTime(2012, 01, 25), Name = "Analysis", Progress = 30 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 25), FinishDate = new DateTime(2012, 01, 28), Name = "Defining Modules", Progress = 10 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 28), FinishDate = new DateTime(2012, 01, 30), Name = "Development Plan", Progress = 10 });
            teams[2].SubItems.Add(Person);

            Person = new Item() { Name = "Karthick" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 29), FinishDate = new DateTime(2012, 02, 02), Name = "Analysis", Progress = 10 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 02), FinishDate = new DateTime(2012, 02, 07), Name = "Module Development" });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 08), FinishDate = new DateTime(2012, 02, 10), Name = "Self Testing" });
            teams[2].SubItems.Add(Person);

            Person = new Item() { Name = "Suyama" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 29), FinishDate = new DateTime(2012, 02, 01), Name = "Analysis", Progress = 10 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 02), FinishDate = new DateTime(2012, 02, 07), Name = "Module Development" });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 08), FinishDate = new DateTime(2012, 02, 10), Name = "Self Testin" });
            teams[2].SubItems.Add(Person);

            Person = new Item() { Name = "Albert" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 11), FinishDate = new DateTime(2012, 02, 14), Name = "Modules Integration" });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 14), FinishDate = new DateTime(2012, 02, 17), Name = "Integration Testing" });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 18), FinishDate = new DateTime(2012, 02, 19), Name = "Completeness" });
            teams[2].SubItems.Add(Person);


            teams.Add(new Item() { Name = "Doc Team" });
            Person = new Item() { Name = "Laura" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 16), FinishDate = new DateTime(2012, 02, 19), Name = "User Guide Development", Progress = 10 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 19), FinishDate = new DateTime(2012, 02, 21), Name = "Publishing User Guide", Progress = 10 });
            teams[3].SubItems.Add(Person);

            Person = new Item() { Name = "Margaret" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 16), FinishDate = new DateTime(2012, 02, 18), Name = "Web Conetent Development", Progress = 10 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 20), FinishDate = new DateTime(2012, 02, 21), Name = "Publishing Web Conetent", Progress = 10 });
            teams[3].SubItems.Add(Person);


            teams.Add(new Item() { Name = "Sales Team" });
            Person = new Item() { Name = "Steven" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 13), FinishDate = new DateTime(2012, 01, 16), Name = "Defining Target", Progress = 80 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 17), FinishDate = new DateTime(2012, 01, 20), Name = "Defining Startegy", Progress = 50 });
            teams[4].SubItems.Add(Person);

            Person = new Item() { Name = "Janet" };
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 01, 21), FinishDate = new DateTime(2012, 01, 25), Name = "Collect Customers list", Progress = 50 });
            Person.InLineItems.Add(new Item() { StartDate = new DateTime(2012, 02, 25), FinishDate = new DateTime(2012, 02, 28), Name = "Contacting Customer" });
            teams[4].SubItems.Add(Person);

            return teams;
        }
    }

    public class Item : INotifyPropertyChanged
    {
        string _Name;
        DateTime _StartDate;
        DateTime _FinishDate;
        double _progress;
        ObservableCollection<Item> _subItems = new ObservableCollection<Item>();
        ObservableCollection<Item> _inLineItems = new ObservableCollection<Item>();

        public Item()
        {
            _subItems.CollectionChanged += ItemsCollectionChanged;
            _inLineItems.CollectionChanged += ItemsCollectionChanged;
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return _Name;
            }
            set
            {
                _Name = value;
                this.RaisePropertyChanged("Name");
            }
        }

        /// <summary>
        /// Gets or sets the start date.
        /// </summary>
        /// <value>The start date.</value>
        public DateTime StartDate
        {
            get
            {
                return _StartDate;
            }
            set
            {
                _StartDate = value;
                this.RaisePropertyChanged("StartDate");
            }
        }

        /// <summary>
        /// Gets or sets the finish date.
        /// </summary>
        /// <value>The finish date.</value>
        public DateTime FinishDate
        {
            get
            {
                return _FinishDate;
            }
            set
            {
                _FinishDate = value;
                this.RaisePropertyChanged("FinishDate");
            }
        }

        /// <summary>
        /// Gets or sets the progress.
        /// </summary>
        /// <value>The progress.</value>
        public double Progress
        {
            get
            {
                return Math.Round(_progress, 2);
            }
            set
            {
                _progress = value;
                this.RaisePropertyChanged("Progress");
            }
        }

        /// <summary>
        /// Gets or sets the sub items.
        /// </summary>
        /// <value>The sub items.</value>
        public ObservableCollection<Item> SubItems
        {
            get
            {
                return _subItems;
            }
            set
            {
                _subItems = value;

                _subItems.CollectionChanged += ItemsCollectionChanged;

                if (value.Count > 0)
                {
                    _subItems.ToList().ForEach(n =>
                    {
                        /// To listen the changes occuring in child task.
                        n.PropertyChanged += ItemPropertyChanged;
                    });
                    UpdateDates();
                }

                this.RaisePropertyChanged("SubItems");
            }
        }

        /// <summary>
        /// Gets or sets the in line items.
        /// </summary>
        /// <value>The in line items.</value>
        public ObservableCollection<Item> InLineItems
        {
            get
            {
                return _inLineItems;
            }
            set
            {
                _inLineItems = value;

                _inLineItems.CollectionChanged += ItemsCollectionChanged;

                if (value.Count > 0)
                {
                    _inLineItems.ToList().ForEach(n =>
                    {
                        /// To listen the changes occuring in child task.
                        n.PropertyChanged += ItemPropertyChanged;
                    });
                    UpdateDates();
                }

                this.RaisePropertyChanged("InLineItems");
            }
        }

        /// <summary>
        /// Itemses the collection changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.Collections.Specialized.NotifyCollectionChangedEventArgs"/> instance containing the event data.</param>
        void ItemsCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Add)
            {
                foreach (Item item in e.NewItems)
                {
                    item.PropertyChanged += ItemPropertyChanged;
                }
            }
            else
            {
                foreach (Item item in e.OldItems)
                    item.PropertyChanged -= ItemPropertyChanged;
            }
            UpdateDates();
        }

        /// <summary>
        /// Items the property changed.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.ComponentModel.PropertyChangedEventArgs"/> instance containing the event data.</param>
        void ItemPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName != null)
                if (e.PropertyName == "StartDate" || e.PropertyName == "FinishDate" || e.PropertyName == "Progress")
                {
                    UpdateDates();
                }
        }

        /// <summary>
        /// Updates the dates.
        /// </summary>
        private void UpdateDates()
        {
            var tempCal = 0d;

            if (_subItems.Count > 0)
            {
                /// Updating the start and end date based on the chagne occur in the date of child task
                StartDate = _subItems.Select(c => c.StartDate).Min();
                FinishDate = _subItems.Select(c => c.FinishDate).Max();
                Progress = (_subItems.Aggregate(tempCal, (cur, task) => cur + task.Progress)) / _subItems.Count;
            }

            if (_inLineItems.Count > 0)
            {
                /// Updating the start and end date based on the chagne occur in the date of child task
                StartDate = _inLineItems.Select(c => c.StartDate).Min();
                FinishDate = _inLineItems.Select(c => c.FinishDate).Max();
                Progress = (_inLineItems.Aggregate(tempCal, (cur, task) => cur + task.Progress)) / _inLineItems.Count;
            }
        }

        #region INotifyPropertyChanged Members

        /// <summary>
        /// Raises the property changed.
        /// </summary>
        /// <param name="propName">Name of the prop.</param>
        void RaisePropertyChanged(string propName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged(this, new PropertyChangedEventArgs(propName));
        }

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion
    }
}
